plus1 = "<font size=\"+1\">"

np <- 100
x <- c(0.5,rep(seq(0,1,length=np),np))
y <- c(0.5,rep(seq(0,1,length=np),each=np))
xy <- data.frame(x=x,y=y)

shinyServer(function(input, output) {
 
    gotLibs <- reactive({
        a<-is.element("ggplot2", installed.packages()[,1])
        b<-is.element("grid", installed.packages()[,1])
        x <- c(a,b, a&b)
        names(x) <- c("ggplot2","grid","both")
        x
    })
    output$checkGraph <- renderText({
        if(input$whichgraph=="base") return("")
        if(gotLibs()[1]) {
            library(ggplot2)
            return("")
        }    
        else return("<h5>Warning: ggplot2 graphics package not found, will use base graphics</h5>")
        NULL       
    })
       
  data <- reactive({
      n <- sample(20:100,1)
      intercept <- runif(1,-5,5)
      slope <- runif(1,-5,5)
      sd <- runif(1,0,2)
      x <- c(0,sort(runif(n-2)),1)
      y <- intercept+slope*x+rnorm(n,0,sd)
      truey <- c(intercept,intercept+slope)
      truey <- (truey-min(y))/diff(range(y))
      intercept <- truey[1]
      slope <- diff(truey)
      y <- (y-min(y))/diff(range(y))      
      list(data.frame(x=x,y=y),par=round(c(intercept, slope),2))
  })
  
  vals <- reactiveValues(
    keeprows = c(FALSE,rep(FALSE, np*np))
  )
  
  observeEvent(input$plot1_click, {
    res <- nearPoints(xy, input$plot1_click, threshold = 5, 
          xvar="x",yvar="y",maxpoints = 1, allRows = TRUE)

    vals$keeprows <- xor(vals$keeprows, res$selected_)
  })


  # Reset all points
  observeEvent(input$exclude_reset, {
    vals$keeprows <- c(FALSE,rep(FALSE, np*np))
  })  
  
  output$plot1 <- renderPlot({
      pickPoints <- xy[vals$keeprows, , drop = FALSE]
    
      if(nrow(pickPoints)==2) {
          slope <- diff(pickPoints[,2])/diff(pickPoints[,1])
          intercept <- pickPoints[1,2]-slope*pickPoints[1,1]
      }    
      else {slope <- 0;intercept <- 0}
      
      co_lm <- coef(lm(y~x, data=data()[[1]]))
      if(input$whichgraph=="base" | !gotLibs()[1]) {
          plot(data()[[1]],pch=20)
          if(nrow(pickPoints)==2) abline(intercept,slope,lwd=2,col="blue")
          if(nrow(pickPoints)>0) points(pickPoints,col="red",cex=1.5,pch="x")
          if(input$showlsr=="Yes")
              abline(co_lm,lwd=2,col="red")       
          if(input$true=="Yes")
              abline(data()[[2]][1],data()[[2]][2],lwd=2)
      }
      else {
            dta <- data()[[1]]
            plt <- ggplot(dta,aes(x,y))+
                   geom_point(size=1.2)
            if(nrow(pickPoints)==2)       
                   plt <- plt+geom_abline(intercept=intercept,slope=slope,size=1.2,colour="blue")
            if(nrow(pickPoints)>0) 
                   plt <- plt+geom_point(data=data.frame(x=pickPoints[,1],y=pickPoints[,2]),aes(x,y),colour="red",size=3)       
            if(input$showlsr=="Yes") 
                plt <- plt+geom_abline(intercept=co_lm[1],slope=co_lm[2],colour="red",size=1.2)
            if(input$true=="Yes") 
                plt <- plt+geom_abline(intercept=data()[[2]][1],slope=data()[[2]][2],colour="black",size=1.2)
            print(plt)    
      } 
  })  
      
  output$text1 <- renderText({ 
      lineA <- ""
      if(input$true=="Yes") {
          cf <- round(data()[[2]],2)
          sgn=ifelse(sign(cf[2])==1,"+","-")
          lineA <- paste("<h4>Population Equation:  y = ",cf[1],sgn,abs(cf[2]),"x</h4>",collapse="")
      }
      
      lineB <- ""
      if(input$showlsr=="Yes") {
          x <- data()[[1]][,1]
          y <- data()[[1]][,2]
          fit <- lm(y~x)
          cf <- round(coef(fit),2)
          r2 <- round(summary(fit)$r.squared*100,1)
          sgn=ifelse(sign(cf[2])==1,"+","-")
          lineB <- paste("<h4>Least Squares Regression Equation:",HTML("y&#770;")," = ",cf[1],sgn,abs(cf[2]),
                    "x (R<sup>2</sup> = ",r2,"%)</h4>",collapse="")
      } 
      
      pickPoints <- xy[vals$keeprows, , drop = FALSE]
    
      if(nrow(pickPoints)==2) {
          slope <- round(diff(pickPoints[,2])/diff(pickPoints[,1]),2)
          intercept <- round(pickPoints[1,2]-slope*pickPoints[1,1],2)
      }    
      else {slope <- 0;intercept <- 0}
      if(nrow(pickPoints)<2) return(HTML("<h4>Pick two points on the line with the mouse</h4>"))
      yhat <- intercept+slope*data()[[1]][,1]
      eps <- data()[[1]][,2]-yhat
      RSS <- sum(eps^2)
      RT <- sum( (data()[[1]][,2]-mean(data()[[1]][,2]))^2)
      r2 <- round((1-RSS/RT)*100,1)
      sgn=ifelse(sign(slope)==1,"+","-")
      lineC <- paste("<h4>User Chosen Equation:",HTML("y&#770;")," = ",intercept,sgn,abs(slope),
                    "x (R<sup>2</sup> = ",r2,"%)</h4>",collapse="")

         
      c(lineC,lineA,lineB)
   })
     
   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#lsr1\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 8/2/2015</h6>")
      
  })     
     output$explainText <- renderText({ 
         "<h5>App generates an example for a least squares problem
         <br>user can pick their own line, then show the least squares regression line and the true population line</h5>" 
     })

})

